==> ltrapi_1.32.17.tar.gz: BUILD/lib/flash/ports/stm32h7xx_spi/flash_iface_stm32h7xx_spi.c <==
#include "flash_iface_stm32h7xx_spi.h"
#include "spi_flash_port_cfg.h"
#include <string.h>

#define FLASH_SPI_FIFO_LEN 16

#define FLASH_SPI          FLASH_SPI_(FLASH_SPI_IFACE_NUM)
#define FLASH_SPI_(num)    FLASH_SPI__( num)
#define FLASH_SPI__(num)   ST_SPI ## num

#define FLASH_SPI_PER_ID         FLASH_SPI_PER_ID_(FLASH_SPI_IFACE_NUM)
#define FLASH_SPI_PER_ID_(num)   FLASH_SPI_PER_ID__(num)
#define FLASH_SPI_PER_ID__(num)  CHIP_PER_ID_SPI ## num



#define FLASH_SPI_INCLK_FREQ               FLASH_SPI_INCLK_FREQ_(FLASH_SPI_IFACE_NUM)
#define FLASH_SPI_INCLK_FREQ_(num)         FLASH_SPI_INCLK_FREQ__(num)
#define FLASH_SPI_INCLK_FREQ__(num)        CHIP_CLK_SPI ## num ## _KER_FREQ



#define FLASH_SPI_CLK_DIV_MIN  (FLASH_SPI_INCLK_FREQ/FLASH_SPI_SCK_FREQ_MAX)

#if FLASH_SPI_CLK_DIV_MIN > 256
    #error max flash freq is out of range
#elif FLASH_SPI_CLK_DIV_MIN > 128
    #define FLASH_SPI_RVAL_MBR 7
    #define FLASH_SPI_DIV_VAL  256
#elif FLASH_SPI_CLK_DIV_MIN > 64
    #define FLASH_SPI_RVAL_MBR 6
    #define FLASH_SPI_DIV_VAL  128
#elif FLASH_SPI_CLK_DIV_MIN > 32
    #define FLASH_SPI_RVAL_MBR 5
    #define FLASH_SPI_DIV_VAL  64
#elif FLASH_SPI_CLK_DIV_MIN > 16
    #define FLASH_SPI_RVAL_MBR 4
    #define FLASH_SPI_DIV_VAL  32
#elif FLASH_SPI_CLK_DIV_MIN > 8
    #define FLASH_SPI_RVAL_MBR 3
    #define FLASH_SPI_DIV_VAL  16
#elif FLASH_SPI_CLK_DIV_MIN > 4
    #define FLASH_SPI_RVAL_MBR 2
    #define FLASH_SPI_DIV_VAL  8
#elif FLASH_SPI_CLK_DIV_MIN > 2
    #define FLASH_SPI_RVAL_MBR 1
    #define FLASH_SPI_DIV_VAL  4
#else
    #define FLASH_SPI_RVAL_MBR 1
    #define FLASH_SPI_DIV_VAL  2
#endif

#define FLASH_SPI_SCK_FREQ       (FLASH_SPI_INCLK_FREQ/FLASH_SPI_DIV_VAL)

#define FLASH_SPI_CS_DELAY_CLK_CNT  (FLASH_SPI_CS_SETUP_TIME_NS_MIN * CHIP_CLK_CPU_FREQ/1000000)

/* количество слов переданных, но не принятых.
 * Нужно для того, чтобы из-за задержки чтения не переполнелась очередь чтения */
static int f_onfly_wrds_cnt = 0;

static t_flash_errs f_send(struct st_flash_iface *iface,
                           const unsigned char* vals, size_t *psize) {
    (void)iface;
    size_t snd_size = *psize;
    while ((snd_size != 0) && ((FLASH_SPI->SR & ST_SPI_SR_TXP) != 0) && (f_onfly_wrds_cnt < FLASH_SPI_FIFO_LEN)) {
        if (vals) {
            FLASH_SPI->TXDR8 = *vals++;
        } else {
            FLASH_SPI->TXDR8 = 0xFF;
        }
        f_onfly_wrds_cnt++;
        snd_size--;
    }
    *psize = snd_size;
    return 0;
}



static t_flash_errs f_recv(struct st_flash_iface *iface,
                           unsigned char* vals, size_t *psize) {
    (void)iface;

    size_t rcv_size = *psize;

    while ((rcv_size!=0) && ((FLASH_SPI->SR & ST_SPI_SR_RXP) != 0)) {
        volatile uint8_t ret;
        ret = FLASH_SPI->RXDR8;

        chip_wdt_reset();
        if (vals != NULL) {
            *vals++ = ret;
        }
        f_onfly_wrds_cnt--;
        rcv_size--;
    }
    *psize = rcv_size;
    return 0;
}



static t_flash_errs f_select(t_flash_iface *iface, size_t total_size) {
    (void)iface;
    (void)total_size;
    f_onfly_wrds_cnt = 0;
    CHIP_PIN_OUT(FLASH_SPI_PIN_CS, 0);
    __DSB();
    chip_delay_clk(FLASH_SPI_CS_DELAY_CLK_CNT);
    __ISB();
    //FLASH_SPI->CR2 = LBITFIELD_SET(ST_SPI_CR2_TSIZE, total_size);
    FLASH_SPI->CR1 |= ST_SPI_CR1_CSTART;
    /* clear data */
    return 0;
}

static t_flash_errs f_unselect(t_flash_iface *iface) {
    (void)iface;
    /* ожидаем завершения передачи всех данных */
    while ((FLASH_SPI->SR & ST_SPI_SR_TXC) == 0) {}
    CHIP_PIN_OUT(FLASH_SPI_PIN_CS, 1);
    __DSB();
    chip_delay_clk(FLASH_SPI_CS_DELAY_CLK_CNT);
    __ISB();
    return 0;
}





t_flash_errs flash_iface_init_stm32h7xx_spi(t_flash_iface *iface) {
    t_flash_errs err = iface==NULL ? FLASH_ERR_INVALID_PARAMS : 0;
    if (!err) {
        chip_per_rst(FLASH_SPI_PER_ID);
        chip_per_clk_en(FLASH_SPI_PER_ID);


        FLASH_SPI->CR1 &= 0xFFFE00FE;
        FLASH_SPI->CR1 |= ST_SPI_CR1_SSI;

        CHIP_PIN_CONFIG(FLASH_SPI_PIN_SCK);
        CHIP_PIN_CONFIG(FLASH_SPI_PIN_MISO);
        CHIP_PIN_CONFIG(FLASH_SPI_PIN_MOSI);
        CHIP_PIN_CONFIG_OUT(FLASH_SPI_PIN_CS, 1);



        FLASH_SPI->CFG1 = (FLASH_SPI->CFG1 & 0x8FA02000)
                | LBITFIELD_SET(ST_SPI_CFG1_MBR, FLASH_SPI_RVAL_MBR)
                | LBITFIELD_SET(ST_SPI_CFG1_FTHLV, 0)
                | LBITFIELD_SET(ST_SPI_CFG1_DSIZE, 7);
        FLASH_SPI->CFG2 = (FLASH_SPI->CFG2 & 0x08017F00)
                | LBITFIELD_SET(ST_SPI_CFG2_AFCNTR, 1)
                | LBITFIELD_SET(ST_SPI_CFG2_SSOE, 0)
                | LBITFIELD_SET(ST_SPI_CFG2_CPOL, FLASH_SPI_MODE_CPOL)
                | LBITFIELD_SET(ST_SPI_CFG2_CPHA, FLASH_SPI_MODE_CPHA)
                | LBITFIELD_SET(ST_SPI_CFG2_MASTER, 1)
                | LBITFIELD_SET(ST_SPI_CFG2_SSM, 1)
                ;


        FLASH_SPI->CR1 |= ST_SPI_CR1_SPE;

        memset(iface, 0, sizeof(t_flash_iface));
        iface->sendrcv_tout = 2;
        iface->select = f_select;
        iface->unselect = f_unselect;
        iface->send = f_send;
        iface->recv = f_recv;
        iface->flush = NULL;

    }
    return  err;
}

t_flash_errs flash_iface_close_stm32h7xx_spi(t_flash_iface *iface) {
    (void)iface;
    FLASH_SPI->CR1 |= ST_SPI_CR1_CSUSP;
    while ((FLASH_SPI->SR & ST_SPI_SR_TXC) == 0) {}
    FLASH_SPI->CR1 &= ~ST_SPI_CR1_SPE;

    chip_per_clk_dis(FLASH_SPI_PER_ID);
    return  0;
}

==> ltrapi_1.32.28.tar.gz: BUILD/lib/flash/ports/stm32h7xx_spi/flash_iface_stm32h7xx_spi.c <==
#include "flash_iface_stm32h7xx_spi.h"
#include "spi_flash_port_cfg.h"
#include <string.h>

#define FLASH_SPI_FIFO_LEN 16

#define FLASH_SPI          FLASH_SPI_(FLASH_SPI_IFACE_NUM)
#define FLASH_SPI_(num)    FLASH_SPI__( num)
#define FLASH_SPI__(num)   ST_SPI ## num

#define FLASH_SPI_PER_ID         FLASH_SPI_PER_ID_(FLASH_SPI_IFACE_NUM)
#define FLASH_SPI_PER_ID_(num)   FLASH_SPI_PER_ID__(num)
#define FLASH_SPI_PER_ID__(num)  CHIP_PER_ID_SPI ## num



#define FLASH_SPI_INCLK_FREQ               FLASH_SPI_INCLK_FREQ_(FLASH_SPI_IFACE_NUM)
#define FLASH_SPI_INCLK_FREQ_(num)         FLASH_SPI_INCLK_FREQ__(num)
#define FLASH_SPI_INCLK_FREQ__(num)        CHIP_CLK_SPI ## num ## _KER_FREQ



#define FLASH_SPI_CLK_DIV_MIN  ((FLASH_SPI_INCLK_FREQ + FLASH_SPI_SCK_FREQ_MAX - 1)/FLASH_SPI_SCK_FREQ_MAX)

#if FLASH_SPI_CLK_DIV_MIN > 256
    #error max flash freq is out of range
#elif FLASH_SPI_CLK_DIV_MIN > 128
    #define FLASH_SPI_RVAL_MBR 7
    #define FLASH_SPI_DIV_VAL  256
#elif FLASH_SPI_CLK_DIV_MIN > 64
    #define FLASH_SPI_RVAL_MBR 6
    #define FLASH_SPI_DIV_VAL  128
#elif FLASH_SPI_CLK_DIV_MIN > 32
    #define FLASH_SPI_RVAL_MBR 5
    #define FLASH_SPI_DIV_VAL  64
#elif FLASH_SPI_CLK_DIV_MIN > 16
    #define FLASH_SPI_RVAL_MBR 4
    #define FLASH_SPI_DIV_VAL  32
#elif FLASH_SPI_CLK_DIV_MIN > 8
    #define FLASH_SPI_RVAL_MBR 3
    #define FLASH_SPI_DIV_VAL  16
#elif FLASH_SPI_CLK_DIV_MIN > 4
    #define FLASH_SPI_RVAL_MBR 2
    #define FLASH_SPI_DIV_VAL  8
#elif FLASH_SPI_CLK_DIV_MIN > 2
    #define FLASH_SPI_RVAL_MBR 1
    #define FLASH_SPI_DIV_VAL  4
#else
    #define FLASH_SPI_RVAL_MBR 0
    #define FLASH_SPI_DIV_VAL  2
#endif

#define FLASH_SPI_SCK_FREQ       (FLASH_SPI_INCLK_FREQ/FLASH_SPI_DIV_VAL)

#define FLASH_SPI_CS_DELAY_CLK_CNT  (FLASH_SPI_CS_SETUP_TIME_NS_MIN * CHIP_CLK_CPU_FREQ/1000000)

/* количество слов переданных, но не принятых.
 * Нужно для того, чтобы из-за задержки чтения не переполнелась очередь чтения */
static int f_onfly_wrds_cnt = 0;

static t_flash_errs f_send(struct st_flash_iface *iface,
                           const unsigned char* vals, size_t *psize) {
    (void)iface;
    size_t snd_size = *psize;
    while ((snd_size != 0) && ((FLASH_SPI->SR & ST_SPI_SR_TXP) != 0) && (f_onfly_wrds_cnt < FLASH_SPI_FIFO_LEN)) {
        if (vals) {
            FLASH_SPI->TXDR8 = *vals++;
        } else {
            FLASH_SPI->TXDR8 = 0xFF;
        }
        f_onfly_wrds_cnt++;
        snd_size--;
    }
    *psize = snd_size;
    return 0;
}



static t_flash_errs f_recv(struct st_flash_iface *iface,
                           unsigned char* vals, size_t *psize) {
    (void)iface;

    size_t rcv_size = *psize;

    while ((rcv_size!=0) && ((FLASH_SPI->SR & ST_SPI_SR_RXP) != 0)) {
        volatile uint8_t ret;
        ret = FLASH_SPI->RXDR8;

        chip_wdt_reset();
        if (vals != NULL) {
            *vals++ = ret;
        }
        f_onfly_wrds_cnt--;
        rcv_size--;
    }
    *psize = rcv_size;
    return 0;
}



static t_flash_errs f_select(t_flash_iface *iface, size_t total_size) {
    (void)iface;
    (void)total_size;
    f_onfly_wrds_cnt = 0;
    CHIP_PIN_OUT(FLASH_SPI_PIN_CS, 0);
    __DSB();
    chip_delay_clk(FLASH_SPI_CS_DELAY_CLK_CNT);
    __ISB();
    //FLASH_SPI->CR2 = LBITFIELD_SET(ST_SPI_CR2_TSIZE, total_size);
    FLASH_SPI->CR1 |= ST_SPI_CR1_CSTART;
    /* clear data */
    return 0;
}

static t_flash_errs f_unselect(t_flash_iface *iface) {
    (void)iface;
    /* ожидаем завершения передачи всех данных */
    while ((FLASH_SPI->SR & ST_SPI_SR_TXC) == 0) {}
    CHIP_PIN_OUT(FLASH_SPI_PIN_CS, 1);
    __DSB();
    chip_delay_clk(FLASH_SPI_CS_DELAY_CLK_CNT);
    __ISB();
    return 0;
}





t_flash_errs flash_iface_init_stm32h7xx_spi(t_flash_iface *iface) {
    t_flash_errs err = iface==NULL ? FLASH_ERR_INVALID_PARAMS : 0;
    if (!err) {
        chip_per_rst(FLASH_SPI_PER_ID);
        chip_per_clk_en(FLASH_SPI_PER_ID);


        FLASH_SPI->CR1 &= 0xFFFE00FE;
        FLASH_SPI->CR1 |= ST_SPI_CR1_SSI;

        CHIP_PIN_CONFIG(FLASH_SPI_PIN_SCK);
        CHIP_PIN_CONFIG(FLASH_SPI_PIN_MISO);
        CHIP_PIN_CONFIG(FLASH_SPI_PIN_MOSI);
        CHIP_PIN_CONFIG_OUT(FLASH_SPI_PIN_CS, 1);



        FLASH_SPI->CFG1 = (FLASH_SPI->CFG1 & 0x8FA02000)
                | LBITFIELD_SET(ST_SPI_CFG1_MBR, FLASH_SPI_RVAL_MBR)
                | LBITFIELD_SET(ST_SPI_CFG1_FTHLV, 0)
                | LBITFIELD_SET(ST_SPI_CFG1_DSIZE, 7);
        FLASH_SPI->CFG2 = (FLASH_SPI->CFG2 & 0x08017F00)
                | LBITFIELD_SET(ST_SPI_CFG2_AFCNTR, 1)
                | LBITFIELD_SET(ST_SPI_CFG2_SSOE, 0)
                | LBITFIELD_SET(ST_SPI_CFG2_CPOL, FLASH_SPI_MODE_CPOL)
                | LBITFIELD_SET(ST_SPI_CFG2_CPHA, FLASH_SPI_MODE_CPHA)
                | LBITFIELD_SET(ST_SPI_CFG2_MASTER, 1)
                | LBITFIELD_SET(ST_SPI_CFG2_SSM, 1)
                ;


        FLASH_SPI->CR1 |= ST_SPI_CR1_SPE;

        memset(iface, 0, sizeof(t_flash_iface));
        iface->sendrcv_tout = 2;
        iface->select = f_select;
        iface->unselect = f_unselect;
        iface->send = f_send;
        iface->recv = f_recv;
        iface->flush = NULL;

    }
    return  err;
}

t_flash_errs flash_iface_close_stm32h7xx_spi(t_flash_iface *iface) {
    (void)iface;
    FLASH_SPI->CR1 |= ST_SPI_CR1_CSUSP;
    while ((FLASH_SPI->SR & ST_SPI_SR_TXC) == 0) {}
    FLASH_SPI->CR1 &= ~ST_SPI_CR1_SPE;

    chip_per_clk_dis(FLASH_SPI_PER_ID);
    return  0;
}
